function [clusters clustCenters] = Clustering(method,data,par1,par2,par3,par4,centers)
% Clustering computes and plots 2D/3D clusters using several methods and parameters
%
% [clusters clustCenters] = Clustering(method,data,par1,par2,par3,par4,centers);
%
% - Input arguments
%     method:         Hierarchical, K-Means, Fuzzy C-Means, Fuzzy Subtractive, Dignet
%     data:           reduced 2D or 3D data
%     par1/2/3/4:     parameters
%     centers         plot cluster centers (0:no, 1:yes)
%
% - Output arguments
%     clusters:       cluster index vector with data length
%     clustCenters:   cluster center coordinates
%
%
% CLASSE Project
% Created by Edgar Ferreira - FCTUC
% edgars@student.dei.uc.pt / edgar.bajouca@gmail.com
% Updated April 20, 2007
%
% Copyright (C) 2007 Edgar Ferreira  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


% Initialize sub plot variable
subPlot = [];

% Switch clustering method
switch method
    case 'Hierarchical'
        nr=par1;        % number of clusters
        met_dist=par2;  % distance metric
        met_link=par3;  % linkage
        tree=par4;      % to create or not dendrogram (0:no, 1:yes)
        centers=0;      % no centers available in this method

        % Depending on tree value, find cluster index and plot dendrogram
        if tree==1
            if strcmp(met_dist,'Minkowski (p=9)')==1
                distance = pdist(data,'minkowski',9);       % create distance matrix
                clusters = clusterdata(data,'distance','minkowski',9,'linkage',met_link,nr);   % create cluster index
            else
                distance = pdist(data,met_dist);            % create distance matrix
                clusters = clusterdata(data,'distance',met_dist,'linkage',met_link,nr);    	% create cluster index
            end
            clustTree = linkage(distance,met_link);         % create linkage tree
            % Create sub plot structure
            subPlot.method = 'dendrogram';
            subPlot.parameter = clustTree;
        else
            if strcmp(met_dist,'Minkowski (p=9)')==1
                clusters = clusterdata(data,'distance','minkowski',9,'linkage',met_link,nr);   % create cluster index
            else
                clusters = clusterdata(data,'distance',met_dist,'linkage',met_link,nr);    	% create cluster index
            end
        end
        tit=['Hierarchical Clustering - Metric: ' met_dist ' - Linkage: ' met_link];
        clustCenters = [];
        
    case 'K-Means'
        nr=par1;        % number of clusters
        distance=par2;  % distance metric
        rep=par3;       % replicates
        sil=par4;       % to create or not silhouette (0:no, 1:yes)

        % Depending on sil value, find cluster index and plot silhouette
        [clusters, clustCenters] = kmeans(data,nr,'replicates',rep,'distance',distance);   % create cluster index

        if sil==1
            % Create sub plot structure
            subPlot.method = 'silhouette';
            subPlot.parameter = distance;
        end
        tit=['K-Means Clustering - Metric: ' distance ' - ' num2str(rep) ' replicates'];

    case 'Fuzzy C-Means'
        nr=par1;        % number of clusters
        options=par2;   % options: [exp max min 0]:
            % exp: exponent for the partition matrix U (default: 2.0)
            % max: maximum number of iterations (default: 100)
            % min: minimum amount of improvement (default: 1e-5)

        % Clustering Using Fuzzy C-Means Clustering
        [clustCenters, U] = fcm(data,nr,options);      % create partition matrix

        % Find cluster index
        [maxU clusters] = max(U);
        clusters=clusters';
        tit=['Fuzzy C-Means Clustering - ' num2str(nr) ' clusters'];

    case 'Fuzzy Subtractive'
        radii=par1;     % range of influence
        options=par2;   % options: [quashFactor acceptRatio rejectRatio 0]:
        % quashFactor: This is the factor used to multiply the radii
        %      values that determine the neighborhood of a cluster center, so as to
        %      quash the potential for outlying points to be considered as part of
        %      that cluster. (default: 1.25)
        % acceptRatio: This sets the potential, as a fraction of the
        %      potential of the first cluster center, above which another data point
        %      will be accepted as a cluster center. (default: 0.5)
        % rejectRatio: This sets the potential, as a fraction of the
        %      potential of the first cluster center, below which a data point will be
        %      rejected as a cluster center. (default: 0.15)

        % Find cluster centers
        [clustCenters, S] = subclust(data,radii,[],options);

        % Define clusters for each point
        temp=dist(data, clustCenters');
        if size(temp,2)>1
            [dummy,i]=min(temp');
            clusters=i';
        else
            clusters=ones(size(data,1),1);
        end
        nr=max(clusters);
        tit=['Fuzzy Subtractive Clustering - Created ' num2str(nr) ' clusters'];

    case 'Dignet'
        %+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        %                   CLUSTERING ALGORITHM DIGNET
        %+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        %Author:
        %      Justo Elias Matheus Valero
        %      matheusj@dei.uc.pt
        %      CISUC- University of Coimbra
        %Adapted by:
        %      Edgar Ferreira - FCTUC
        %This script implements the cluster algorithm Dignet

        datos=data;     % reduced data
        Thetat=par1;    % threshold
        met=par2;       % similarity metric

        %+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        % Definition of Parameters

        % Definition of the Threshold
        % Thetat=0.45;    % an input variable

        % Get the variable of the input data;
        % nfd= Number of data rows;
        % ncd= Number of data columns
        [nfd,ncd]=size(datos);

        % Vector for the classification of the input file
        recall_data=zeros(nfd,1);
        % Define the depth of the attraction wels
        profundidad=zeros(1,50);
        % Internal counter
        cceros=0;
        % Number of centers
        nc=1;
        clear Simil;

        %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        % Get information from data matrix

        %Repeat for all the input rows
        for i=1 : nfd
            % Getting the ith rows of the matrix datos
            vector=datos(i,:);
            % For the first iteration
            if (i==1)
                % New center equal to the first row
                C(nc,:)=vector;
                % Initialization of the depth
                profundidad(nc)=1;
            else
                %Calculating the similarities for all the centers
                %Repeat for all the centers
                for j=1 : nc
                    %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                    % Calculating the similarity angle (added by EF)

                    if strcmp(met,'Angular')==1
                        normacentro=norm(C(j,:));     % Norm of the center
                        normapatron=norm(vector);     % Norm of the input the vector
                        %Calculating the internal product between the center and the vector (added by EF)
                        V1=C(j,:);
                        V2=vector;
                        if length(V1)~=length(V2)
                            fprintf('Error! Vectors must be the same lenght.\n');
                        else
                            q=1;
                            R=0;
                            while q<=length(V1)
                                R=R+V1(q)*V2(q);
                                q=q+1;
                            end
                        end
                        producto=R;
                        clear V1 V2 q R

                        Simil(j)=acos(producto/(normacentro*normapatron));       % Angular Similarity
                    elseif strcmp(met,'Chebychev')==1
                        Simil(j)=max(abs(C(j,:)-vector));                        % Hypercubic Similarity
                    elseif strcmp(met,'Euclidean')==1
                        Simil(j)=sqrt(sum((C(j,:)-vector).^2));                  % Euclidean Similarity
                    elseif strcmp(met,'Cityblock')==1
                        Simil(j)=sum(abs(C(j,:)-vector));                        % Manhattan Similarity
                    end
                    %+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                end     %End  for j

                %Calculating the state of the update for the centers
                %Condition for one attraction well
                if nc ==1
                    %Min similaridad
                    thetamin=Simil(nc);
                    %Indicating the winning center
                    donde=1;
                else
                    %Condition for multiple attraction wells
                    [thetamin,donde]=min(Simil);
                end

                %Flag to know if the input row was absorved by any attraction well
                nuevo_nodo=1;
                %If the minimum similarity is lower than the threshold
                if (thetamin<Thetat)
                    %actualizar store the status of the attraction well
                    actualizar(donde)=1;
                    %The pattern felt in an attraction well
                    nuevo_nodo=0;
                else
                    %The pattern did not fall in any attraction well
                    actualizar(donde)=0;
                end

                %Updating the update status for all the wells
                for j=1 : nc
                    %Condition for all the well except the winning well
                    if (~(j == donde))
                        %The pattern is in the attraction of the well j but it did not wing
                        if (Simil(j) < Thetat)
                            actualizar(j)=-1;
                        else
                            %The pattern did not fall in any attraction well
                            actualizar(j)=0;
                        end                               %End (Simil(j)<Thetat)

                    end                                 %End (j<>donde)
                end     %End for j

                %**********************************************************
                %End of the update status
                %*********************************************************

                %+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                %Updating the depth of the attraction well
                for j=1 : nc
                    %Storing the old depth for the well j
                    aux_profunidad=profundidad(j);
                    %Updating the depth
                    profundidad(j)=profundidad(j)+actualizar(j);
                    %Updating the attraction well

                    if ~(profundidad(j)==0)                     %Avoiding  0 division
                        C(j,:)=actualizar(j)/profundidad(j).*vector+aux_profunidad/profundidad(j).*C(j,:);
                    else
                        cceros=cceros+1;                           %Internal conter
                    end

                end %for j

                %Let see if a new attraction well is needed
                %If the pattern did not fall in any attraction well
                if (nuevo_nodo==1)
                    %pause
                    %Increase the amount of atrraction wells
                    nc=nc+1;
                    %New atrraction well is equal to the input pattern
                    C(nc,:)=vector;
                    %Initialzation of the depth for the attraction well
                    profundidad(nc)=1;

                end     %End (nuevo_nodo)
            end         %End else
        end             %End for i

        %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        %Classification of the input patterns
        membership_value=zeros(nfd,nc);
        %Repeat for all the input patterns

        for i=1 : nfd
            %Getting the ith row
            vector=datos(i,:);
            %Repeat for the centers found
            for j=1 : nc
                %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                % Calculating the similarity angle (added by EF)

                if strcmp(met,'Angular')==1
                    normacentro=norm(C(j,:));     % Norm of the center
                    normapatron=norm(vector);     % Norm of the input the vector
                    %Calculating the internal product between the center and the vector (added by EF)
                    V1=C(j,:);
                    V2=vector;
                    if length(V1)~=length(V2)
                        fprintf('Error! Vectors must be the same lenght.\n');
                    else
                        q=1;
                        R=0;
                        while q<=length(V1)
                            R=R+V1(q)*V2(q);
                            q=q+1;
                        end
                    end
                    producto=R;
                    clear V1 V2 q R

                    Simil(j)=acos(producto/(normacentro*normapatron));       % Angular Similarity
                elseif strcmp(met,'Chebychev')==1
                    Simil(j)=max(abs(C(j,:)-vector));                        % Hypercubic Similarity
                elseif strcmp(met,'Euclidean')==1
                    Simil(j)=sqrt(sum((C(j,:)-vector).^2));                  % Euclidean Similarity
                elseif strcmp(met,'Cityblock')==1
                    Simil(j)=sum(abs(C(j,:)-vector));                        % Manhattan Similarity
                end

                %% Storing the similarity of the ith pattern with the jth atraction well
                membership_value(i,j)=Simil(j);
            end
            [thetamin,donde]=min(Simil);
            %Classification of the input pattern
            recall_data(i)=donde;
        end

        %Prevent centers without any vector match
        clusters=zeros(size(recall_data));      % cluster index (added by EF)
        clustCenters=[];        % initialize cluster centers
        nr=0;
        for i=1:max(recall_data)
            indexes=find(recall_data==i);
            if ~isempty(indexes)
                nr=nr+1;
                clusters(indexes)=nr;
                clustCenters=[clustCenters; C(i,:)];
            end
        end
        tit=['Dignet Clustering - Metric: ' met ' - Created ' num2str(nr) ' clusters'];
end

%% Plot Clusters (and Centers)

PlotClusters(data, clusters, [method ' Clustering'], tit, subPlot)

%Plot centers --> colored cross inside a black circle
if centers==1
    cm=colormap;
    j=1;
    k=1;
    if nr<64; k=floor(64/nr); end   % color step
    for i = 1:nr
        if j>64; j=1; end
        if size(data,2)>2
            plot3(clustCenters(i,1),clustCenters(i,2),clustCenters(i,3),'x','color',cm(j,:),'markersize',10,'LineWidth',2);
            plot3(clustCenters(i,1),clustCenters(i,2),clustCenters(i,3),'ko','markersize',10,'LineWidth',2);
        elseif size(data,2)==2
            plot(clustCenters(i,1),clustCenters(i,2),'x','color',cm(j,:),'markersize',10,'LineWidth',2);
            plot(clustCenters(i,1),clustCenters(i,2),'ko','markersize',10,'LineWidth',2);
        end
        j=j+k;
    end
end
return
