function h=MovableLine(action,tag,direction,movecall,upcall,color,yposition)
%MovableLine places a mouse-movable line on the current axis for current data
%
%  Format: h=MovableLine(action,tag,direction,movecall,upcall,color,yposition);
%
%    -action:
%       (handle) = handle of data line or axis to associate MovableLine with
%           This creates the line, and requires tag and direction.
%
%       'mousedown' sets a flag in userdata of line and figure window
%				properties
%       'mouseup'   resets flag and figure properties to null states
%       'mousego'   moves MovableLine
%       'delete'  erases MovableLine
%       'recenter'   recenters MovableLine
%             (all above actions require tag as well)
%
%    -tag: tag of line (if [], default='MovableLine')
%
%	  -direction: 'v' vertical or 'h' horizontal (if [], default='h')
%       For a horiz line, MovableLine is placed at mean of data or ylim
%       For a vert line, MovableLine is placed at the current minimum xlim
%
%    -movecall (optional): additional callback that can be used
%        with 'mousego' (default='')
%
%    -upcall (optional): additional callback that can be used
%        with 'mouseup' (default='')
%
%    -color (optional): color of line (default=[0 0 0])
% 
%    -yposition (optional): vertical position of line (default=[])
%
% Original function named 'testline' and created by
% Bill Miller
% Updated April 10, 2001
%
% 'MovableLine' is an costumized function created by
% Edgar Ferreira, edgars@student.dei.uc.pt / edgar.bajouca@gmail.com
% FCTUC/CISUC, Coimbra, Portugal
% CLASSE Project
% Updated March 12, 2007
%
% Copyright (C) 2007 Edgar Ferreira  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


if nargin<1
    disp(' MovableLine: need a line or axis handle');
    return
end

if isnumeric(action)
    if ~exist('movecall','var'),movecall='';end
    if ~exist('upcall','var'),upcall='';end
    if ~exist('tag','var')
        tag='MovableLine';
    elseif isempty(tag)
        tag='MovableLine';
    end
    if ~exist('direction','var')
        direction='h';
    elseif isempty(direction)
        direction='h';
    end
    if ~exist('color','var')
        color=[0 0 0];
    elseif isempty(direction)
        color=[0 0 0];
    end
    if ~exist('yposition','var')
        yposition=[];
    end

    li=findobj('parent',gca,'tag',tag);
    if li>0
        h=0;
    else
        h=CreateLine(action,tag,direction,movecall,upcall,color,yposition);
    end
    return
end

switch(action)
    case 'mousedown'
        li=findobj('parent',gca,'tag',tag);
        userdata=get(li,'userdata');
        userdata{1}=1;
        set(li,'userdata',userdata);
        cmd1=['MovableLine(''mousego'',''' tag ''');'];
        cmd2=['MovableLine(''mouseup'',''' tag ''');'];
        set(gcf,'WindowButtonMotionFcn',cmd1);
        set(gcf,'WindowButtonUpFcn',cmd2);
        drawnow

    case 'mousego'
        MoveLine(tag,action)

    case 'mouseup'
        DoMouseUp(tag)

    case 'delete'
        li=findobj('parent',gca,'tag',tag);
        set(li,'erasemode','normal')
        delete(li)
        set(gcf,'WindowButtonMotionFcn','');
        set(gcf,'WindowButtonUpFcn','');

    case 'recenter'
        MoveLine(tag,action)
end


%--------------
function li=CreateLine(handle,tag,direction,movecall,upcall,color,yposition)
hold on

tp=get(handle,'type');
switch tp
    case 'axes'
        axhnd=handle;
        xlim=get(axhnd,'xlim');
        ylim=get(axhnd,'ylim');
        ypos=mean(ylim);
        xpos=mean(xlim);
    case 'line'
        axhnd=get(handle,'parent');
        xlim=get(axhnd,'xlim');
        ylim=get(axhnd,'ylim');
        xpos=mean(xlim);
        y=get(handle,'ydata');
        ypos=mean(y);

    otherwise
        disp('  MovableLine: action must be handle to axes or line')
        return
end

cmd1=['MovableLine(''mousedown'',''' tag ''');'];

switch direction
    case 'h'
        xdata=xlim;
        ydata=[ypos ypos];
    case 'v'
        xdata=[xpos xpos];
        ydata=ylim;
    otherwise
        disp('  MovableLine: direction must be ''h'' or ''v'' ')
        return
end

% if y-position was defined at function call
if ~isempty(yposition)
    ydata=[yposition yposition];
end
li=line(xdata,ydata, ...
    'color',color, ...
    'linewidth', 1.5, ...
    'tag',tag, ...
    'erasemode','xor', ...
    'userdata',{0, direction, movecall, upcall},...
    'ButtonDownFcn',cmd1);

%--------------
function MoveLine(tag,action)

li=findobj('parent',gca,'tag',tag);

userdata=get(li,'userdata');
state=userdata{1};

xlim=get(gca,'xlim');
ylim=get(gca,'ylim');

switch action
    case 'recenter'
        xpos=mean(xlim);
        ypos=mean(ylim);
    case 'mousego'
        if state==0,return,end
        pt=get(gca,'currentpoint');
        xpos=pt(1,1);
        ypos=pt(1,2);
end


switch userdata{2}
    case 'h'
        if ypos > ylim(2)
            ypos=ylim(2);
        elseif ypos<ylim(1)
            ypos=ylim(1);
        end
        set(li,'ydata',[ypos ypos])
        drawnow
    case 'v'
        if xpos > xlim(2)
            xpos=xlim(2);
        elseif xpos<xlim(1)
            xpos=xlim(1);
        end
        set(li,'xdata',[xpos xpos])
        drawnow
    otherwise
end

movecall=userdata{3};
eval(movecall)

%--------------
function DoMouseUp(tag)

li=findobj('parent',gca,'tag',tag);
userdata=get(li,'userdata');
state=userdata{1};
if state==0,return,end

userdata{1}=0;
set(li,'userdata',userdata)
set(gcf,'WindowButtonMotionFcn','');
set(gcf,'WindowButtonUpFcn','');
drawnow

upcall=userdata{4};
eval(upcall)