function fitness = OptimizationFitnessGA(Yvec)
% OptimizationFitnessGA calculates fitness of given 2D or 3D representative
% vector
%
% fitness = OptimizationFitnessGA(Yvec);
%
% - Input arguments
%     Yvec:         extended vector representing a 2D or 3D coordinates
%                   matrix: [x1 y1 z1 x2 y2 z2 ... xn yn zn]
%
% - Output arguments
%     fitness:      fitness value computed using given criterion
%
% - Global variables
%     dimension:                    original matrix dimension (2 or 3)
%     pdist_metric:                 distance metric
%     fitnessCriterion:             criterion function for computing fitness
%     optimizationDissimilarities:  original data distance vector
%
%
% CLASSE Project
% Created by Edgar Ferreira - FCTUC
% edgars@student.dei.uc.pt / edgar.bajouca@gmail.com
% Updated June 15, 2007
%
% Copyright (C) 2007 Edgar Ferreira  FCTUC Coimbra, Portugal
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.


% Get global variables
global dimension
global pdist_metric
global fitnessCriterion
global optimizationDissimilarities
weights=1;

% Convert input vector to coordinates matrix
n=length(Yvec)/dimension;
Y=reshape(Yvec,dimension,n)';

% Set criterion function
funNames = {'stress','sstress','metricstress','metricsstress','sammon','strain'};
i = strmatch(lower(fitnessCriterion), funNames);
switch i
    case 1 % 'stress'
        stressFun = @stressCrit;
        metric = false; strain = false;
    case 2 % 'sstress'
        stressFun = @sstressCrit;
        metric = false; strain = false;
    case 3 % 'metricstress'
        stressFun = @metricStressCrit;
        metric = true; strain = false;
    case 4 % 'metricsstress'
        stressFun = @metricSStressCrit;
        metric = true; strain = false;
    case 5 % 'sammon'
        if any(optimizationDissimilarities <= 0)
            error('stats:mdscale:NonpositiveDissimilarities', ...
                'D must contain strictly positive values for Sammon mapping.');
        end
        stressFun = @sammonCrit;
        metric = true; strain = false;
    case 6 % 'strain'
        stressFun = @strainCrit;
        metric = true; strain = true;
end

if strain
    % Strain uses transformed dissimilarities, and needs them as a
    % square matrix.  It's convenient to do that beforehand here.
    dissimilarities = squareform(-0.5 .* (optimizationDissimilarities.^2));
else
    dissimilarities = optimizationDissimilarities;
end

% Get distances for given matrix
if(strcmp(pdist_metric(1:min(9,length(pdist_metric))),'Minkowski'))
    distances = pdist(Y,'Minkowski',str2double(pdist_metric(10)));
else
    distances = pdist(Y,pdist_metric);
end


% For metric scaling, there are no disparities, but for convenience in
% the code, set them equal to the dissimilarities.
if metric
    disparities = dissimilarities;
else
    % For non-metric scaling, compute disparities as the values closest to
    % the current interpoint distances, in the least squares sense, while
    % constrained to be monotonic in the given dissimilarities.
    
    % Keep the configuration on the same scale as the dissimilarities.
    % The non-metric forms of Stress are invariant to scale.
    scale = max(dissimilarities)/max(distances);
    Y = Y * scale;
    distances = distances * scale;

    disparities = lsqisotonic(dissimilarities, distances);
end

fitness = feval(stressFun, Y, disparities, weights, pdist_metric);
return

%==========================================================================

function [S,grad] = stressCrit(Y,disparities,weights,pdist_metric)
%STRESS Stress criterion for nonmetric multidimensional scaling.

% The Euclidean norm of the differences between the distances and the
% disparities, normalized by the Euclidean norm of the distances.
%
% Zero weights are ok as long as the corresponding dissimilarities are
% finite.  However, any zero distances will throw an error:  this criterion
% is not differentiable when two points are coincident - it involves, in
% effect, abs(Y(i,k)-Y(j,k)).  This does sometimes happen, even though a
% configuration that is a local minimizer cannot have coincident points.

%    distances = pdist(Y);
    if(length(pdist_metric)>9)
        if(strcmp(pdist_metric(1:9),'Minkowski'))
            distances = pdist(Y,'Minkowski',str2num(pdist_metric(10)));
        else             
            distances = pdist(Y,pdist_metric);
        end
    else
        distances = pdist(Y,pdist_metric);
    end


diffs = distances - disparities;
sumDiffSq = sum(weights.*diffs.^2);
sumDistSq = sum(weights.*distances.^2);
S = sqrt(sumDiffSq ./ sumDistSq);
if nargout > 1
    [n,p] = size(Y);
    if sumDiffSq > 0
        if all(distances > 0)
            dS = squareform(weights.* ...
                     (diffs./sumDiffSq - distances./sumDistSq) ./ distances);
            repcols = zeros(1,n);
            for i = 1:p
                repcols = repcols + 1;
                dY = Y(:,repcols) - Y(:,repcols)';
                grad(:,i) = sum(dS.*dY,2) .* S;
            end
        else
            error('stats:mdscale:ColocatedPoints', ...
                  ['Points in the configuration have co-located.  Try a' ...
                   ' different\nstarting point, or use a different criterion.']);
        end
    else
        grad = repmat(0,n,p);
    end
end


%==========================================================================

function [S,grad] = metricStressCrit(Y,dissimilarities,weights,pdist_metric)
%METRICSTRESS Stress criterion for metric MDS.

% The Euclidean norm of the differences between the distances and the
% dissimilarities, normalized by the Euclidean norm of the dissimilarities.
%
% Zero weights are ok as long as the corresponding dissimilarities are
% finite.  However, any zero distances will throw an error:  this criterion
% is not differentiable when two points are coincident - it involves, in
% effect, abs(Y(i,k)-Y(j,k)).  This does sometimes happen, even though a
% configuration that is a local minimizer cannot have coincident points.

%    distances = pdist(Y);
    if(length(pdist_metric)>9)
        if(strcmp(pdist_metric(1:9),'Minkowski'))
            distances = pdist(Y,'Minkowski',str2num(pdist_metric(10)));
        else             
            distances = pdist(Y,pdist_metric);
        end
    else
        distances = pdist(Y,pdist_metric);
    end


diffs = distances - dissimilarities;
sumDiffSq = sum(weights.*diffs.^2);
sumDissSq = sum(weights.*dissimilarities.^2);
S = sqrt(sumDiffSq./sumDissSq);

if nargout > 1
    [n,p] = size(Y);
    if sumDiffSq > 0
        if all(distances > 0)
            dS = squareform(weights.*diffs ./ distances);
            repcols = zeros(1,n);
            for i = 1:p
                repcols = repcols + 1;
                dY = Y(:,repcols) - Y(:,repcols)';
                grad(:,i) = sum(dS.*dY,2) ./ (sumDissSq.*S);
            end
        else
            error('stats:mdscale:ColocatedPoints', ...
                  ['Points in the configuration have co-located.  Try a' ...
                   ' different\nstarting point, or use a different criterion.']);
        end
    else
        grad = repmat(0,n,p);
    end
end


%==========================================================================

function [S,grad] = sstressCrit(Y,disparities,weights,pdist_metric)
%SSTRESS Squared stress criterion for nonmetric multidimensional scaling.

% The Euclidean norm of the differences between the squared distances and
% the squared disparities, normalized by the Euclidean norm of the squared
% distances.
%
% Zero weights are ok as long as the corresponding dissimilarities are
% finite.  Zero distances are also OK:  this criterion is differentiable
% even when two points are coincident.

% The normalization used here is sum(distances.^4).  Some authors instead
% use sum(disparities.^4).
   
%    distances = pdist(Y);
    if(length(pdist_metric)>9)
        if(strcmp(pdist_metric(1:9),'Minkowski'))
            distances = pdist(Y,'Minkowski',str2num(pdist_metric(10)));
        else             
            distances = pdist(Y,pdist_metric);
        end
    else
        distances = pdist(Y,pdist_metric);
    end

    
diffs = distances.^2 - disparities.^2;
sumDiffSq = sum(weights.*diffs.^2);
sumDist4th = sum(weights.*distances.^4);
S = sqrt(sumDiffSq ./ sumDist4th);
if nargout > 1
    [n,p] = size(Y);
    if sumDiffSq > 0
        dS = squareform(weights.* (diffs./sumDiffSq - (distances.^2)./sumDist4th));
        repcols = zeros(1,n);
        for i = 1:p
            repcols = repcols + 1;
            dY = Y(:,repcols) - Y(:,repcols)';
            grad(:,i) = sum(dS.*dY,2) .* S .* 2;
        end
    else
        grad = repmat(0,n,p);
    end
end


%==========================================================================

function [S,grad] = metricSStressCrit(Y,dissimilarities,weights,pdist_metric)
%METRICSSTRESS Squared stress criterion for metric MDS.

% The Euclidean norm of the differences between the squared distances and
% the squared dissimilarities, normalized by the Euclidean norm of the
% squared dissimilarities.
%
% Zero weights are ok as long as the corresponding dissimilarities are
% finite.  Zero distances are also OK:  this criterion is differentiable
% even when two points are coincident.
    
%    distances = pdist(Y);
    if(length(pdist_metric)>9)
        if(strcmp(pdist_metric(1:9),'Minkowski'))
            distances = pdist(Y,'Minkowski',str2num(pdist_metric(10)));
        else             
            distances = pdist(Y,pdist_metric);
        end
    else
        distances = pdist(Y,pdist_metric);
    end

    
diffs = distances.^2 - dissimilarities.^2;
sumDiffSq = sum(weights.*diffs.^2);
sumDiss4th = sum(weights.*dissimilarities.^4);
S = sqrt(sumDiffSq./sumDiss4th);
if nargout > 1
    [n,p] = size(Y);
    if sumDiffSq > 0
        dS = squareform(weights.*diffs);
        repcols = zeros(1,n);
        for i = 1:p
            repcols = repcols + 1;
            dY = Y(:,repcols) - Y(:,repcols)';
            grad(:,i) = 2 .* sum(dS.*dY,2) ./ (sumDiss4th.*S);
        end
    else
        grad = repmat(0,n,p);
    end
end


%==========================================================================

function [S,grad] = sammonCrit(Y,dissimilarities,weights,pdist_metric)
%SAMMON Sammon mapping criterion for metric multidimensional scaling.

% The sum of the scaled, squared differences between the distances and
% the dissimilarities, normalized by the sum of the dissimilarities. 
% The squared differences are scaled by the dissimilarities before summing.
%
% Zero weights are ok as long as the corresponding dissimilarities are
% finite.  However, zero distances or dissimilarities will cause problems.

%    distances = pdist(Y);
    if(length(pdist_metric)>9)
        if(strcmp(pdist_metric(1:9),'Minkowski'))
            distances = pdist(Y,'Minkowski',str2num(pdist_metric(10)));
        else             
            distances = pdist(Y,pdist_metric);
        end
    else
        distances = pdist(Y,pdist_metric);
    end

 
diffs = distances - dissimilarities;
sumDiffSq = sum(weights.*diffs.^2 ./ dissimilarities);
sumDiss = sum(weights.*dissimilarities);
S = sumDiffSq ./ sumDiss;
if nargout > 1
    [n,p] = size(Y);
    if sumDiffSq > 0
        if all(distances > 0)
            dS = squareform(weights.*diffs ./ (distances.*dissimilarities));
            repcols = zeros(1,n);
            for i = 1:p
                repcols = repcols + 1;
                dY = Y(:,repcols) - Y(:,repcols)';
                grad(:,i) = 2 .* sum(dS.*dY,2) ./ sumDiss;
            end
        else
            error('stats:mdscale:ColocatedPoints', ...
                  ['Points in the configuration have co-located.  Try a' ...
                   ' different\nstarting point, or use a different criterion.']);
        end
    else
        grad = repmat(0,n,p);
    end
end


%==========================================================================

function [S,grad] = strainCrit(Y,A,weights,pdist_metric)
%STRAIN Strain criterion for metric multidimensional scaling.

% Let Y be a minimizer of norm(A-Yc*Yc'), where Yc = (I-ones(n)/n)*Y = P*Y
% (i,e,, Y centered at the origin), and A = -0.5*(D.^2).  Then X = P*Y is a
% minimizer of norm(B-X*X'), where B = P*A*P (the two quantities differ by
% norm(B-A), a constant).  This is exactly the criterion that is minimized
% in CMDS, but now we have a form that is easier to weight.
%
% Zero weights are ok as long as the corresponding dissimilarities are
% finite.

[n,p] = size(Y);
Yc = Y - repmat(mean(Y,1),n,1);
diffs = weights.*(A - Yc*Yc') ./ sum(weights(:));
S = norm(diffs,'fro');
if nargout > 1
    repcols = zeros(1,n);
    for i = 1:p
        repcols = repcols + 1;
        dSrows = diffs .* Yc(:,repcols);
        dScols = diffs .* Yc(:,repcols)';
        grad(:,i) = (sum(sum(dSrows+dScols,2),1)./n - 2.*sum(dScols,2)) ./ S;
    end
end

%==========================================================================

function yhat = lsqisotonic(x,y,w)
%LSQISOTONIC Isotonic least squares.
%   YHAT = LSQISOTONIC(X,Y) returns a vector of values that minimize the
%   sum of squares (Y - YHAT).^2 under the monotonicity constraint that
%   X(I) > X(J) => YHAT(I) >= YHAT(J), i.e., the values in YHAT are
%   monotonically non-decreasing with respect to X (sometimes referred
%   to as "weak monotonicity").  LSQISOTONIC uses the "pool adjacent
%   violators" algorithm.
%
%   If X(I) == X(J), then YHAT(I) may be <, ==, or > YHAT(J) (sometimes
%   referred to as the "primary approach").  If ties do occur in X, a plot
%   of YHAT vs. X may appear to be non-monotonic at those points.  In fact,
%   the above monotonicity constraint is not violated, and a reordering
%   within each group of ties, by ascending YHAT, will produce the desired
%   appearance in the plot.
%
%   YHAT = LSQISOTONIC(X,Y,W) performs weighted isotonic regression using
%   the non-negative weights in W.

%   Copyright 2003-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.4 $  $Date: 2004/07/28 04:39:23 $

%   References:
%      [1] Kruskal, J.B. (1964) "Nonmetric multidimensional scaling: a
%          numerical method", Psychometrika 29:115-129.
%      [2] Cox, R.F. and Cox, M.A.A. (1994) Multidimensional Scaling,
%          Chapman&Hall.

n = numel(x);
if nargin<3
    yclass = superiorfloat(x,y);
else
    yclass = superiorfloat(x,y,w);
end

% Sort points ascending in x, break ties with y.
[xyord,ord] = sortrows([x(:) y(:)]); iord(ord) = 1:n;
xyord = double(xyord);

% Initialize fitted values to the given values.
yhat = xyord(:,2);

block = 1:n;
if (nargin == 3) && ~isempty(w)
    w = double(w(:)); w = w(ord); % reorder w as a column

    % Merge zero-weight points with preceeding pos-weighted point (or
    % with the following pos-weighted point if at start).
    posWgts = (w > 0);
    if any(~posWgts)
        idx = cumsum(posWgts); idx(idx == 0) = 1;
        w = w(posWgts);
        yhat = yhat(posWgts);
        block = idx(block);
    end

else
    w = ones(size(yhat));
end

while true
    % If all blocks are monotonic, then we're done.
    diffs = diff(yhat);
    if all(diffs >= 0), break; end

    % Otherwise, merge blocks of non-increasing fitted values, and set the
    % fitted value within each block equal to a constant, the weighted mean
    % of values in that block.
    idx = cumsum([1; (diffs>0)]);
    sumyhat = accumarray(idx,w.*yhat);
    w = accumarray(idx,w);
    yhat = sumyhat ./ w;
    block = idx(block);
end

% Broadcast merged blocks out to original points, and put back in the
% original order and shape.
yhat = yhat(block);
yhat = reshape(yhat(iord), size(y));
if isequal(yclass,'single')
    yhat = single(yhat);
end

